<?php
// Copyright (C) <2015-present>  <it-novum GmbH>
//
// This file is licensed under the terms of the openITCOCKPIT Enterprise Edition license agreement.
// The license agreement and license key were sent with the order confirmation.


namespace ORACLEModule\Lib\InitialDatabase;

use App\Model\Table\CommandsTable;
use App\Model\Table\ServicetemplatesTable;
use App\Model\Table\WizardAssignmentsTable;
use itnovum\openITCOCKPIT\InitialDatabase\Importer;

/**
 * Class TemplateImport
 * @package ORACLEModule\Lib\InitialDatabase
 */
class TemplateImport extends Importer {
    /**
     * @var CommandsTable
     */
    private $CommandsTable;

    /**
     * @var ServicetemplatesTable
     */
    private $ServicetemplatesTable;

    /**
     * @var WizardAssignmentsTable
     */
    private $WizardAssignmentsTable;

    /**
     * @var array
     */
    private $mapping = [];

    /**
     * TemplateImport constructor.
     * @param CommandsTable $CommandsTable
     * @param ServicetemplatesTable $ServicetemplatesTable
     * @param WizardAssignmentsTable $WizardAssignmentsTable
     */
    public function __construct(CommandsTable $CommandsTable, ServicetemplatesTable $ServicetemplatesTable, WizardAssignmentsTable $WizardAssignmentsTable) {
        $this->CommandsTable = $CommandsTable;
        $this->ServicetemplatesTable = $ServicetemplatesTable;
        $this->WizardAssignmentsTable = $WizardAssignmentsTable;
    }

    /**
     * @return bool
     */
    public function import() {
        $data = $this->getData();
        foreach ($data['Commands'] as $command) {
            if (isset($command['uuid']) && !$this->CommandsTable->existsByUuid($command['uuid'])) {
                $entity = $this->CommandsTable->newEntity($command);
                $this->CommandsTable->save($entity);
            }
        }
        $this->mapping['servicetemplates'] = [];

        foreach ($data['Servicetemplates'] as $servicetemplate) {
            if ($this->ServicetemplatesTable->existsByUuid($servicetemplate['uuid'])) {
                $existingServicetemplate = $this->ServicetemplatesTable->getServicetemplateByUuid($servicetemplate['uuid']);
                $this->mapping['servicetemplates'][$servicetemplate['uuid']] = $existingServicetemplate['Servicetemplate']['id'];
                //skip current template if it already exists
                continue;
            }
            if (isset($servicetemplate['uuid']) && !$this->ServicetemplatesTable->existsByUuid($servicetemplate['uuid'])) {
                if (isset($servicetemplate['command_id']) && $this->CommandsTable->existsByUuid($servicetemplate['command_id'])) {
                    $command = $this->CommandsTable->getCommandByUuid($servicetemplate['command_id'], true, false)[0];
                    $servicetemplate['command_id'] = $command['id'];
                    if (!empty($servicetemplate['servicetemplatecommandargumentvalues']) && !empty($command['commandarguments'])) {
                        foreach ($servicetemplate['servicetemplatecommandargumentvalues'] as $templateArgumentKey => $templateArgumentValue) {
                            $servicetemplate['servicetemplatecommandargumentvalues'][$templateArgumentKey] = [
                                'commandargument_id' => $this->getCommandArgumentIdByName($templateArgumentValue['commandargument_id'], $command['commandarguments']),
                                'value'              => $templateArgumentValue['value'],
                            ];
                        }
                    }
                    $entity = $this->ServicetemplatesTable->newEntity($servicetemplate);
                    $this->ServicetemplatesTable->save($entity);
                    $this->mapping['servicetemplates'][$entity->get('uuid')] = $entity->get('id');
                }
            }
        }

        if (!empty($data['Wizardassignment'])) {
            $wizardassignmentEntity['uuid'] = $data['Wizardassignment']['uuid'];
            $wizardassignmentEntity['type_id'] = $data['Wizardassignment']['type_id'];
            foreach ($data['Wizardassignment']['servicetemplates']['_ids'] as $servicetemplateUuid) {
                $servicetemplate = $this->ServicetemplatesTable->getServicetemplateByUuid($servicetemplateUuid);
                if (isset($servicetemplate['Servicetemplate']) && isset($servicetemplate['Servicetemplate']['id'])) {
                    $wizardassignmentEntity['servicetemplates']['_ids'][] = $servicetemplate['Servicetemplate']['id'];
                }
            }
            if (!$this->WizardAssignmentsTable->existsByUuidAndTypeId($wizardassignmentEntity['uuid'], $wizardassignmentEntity['type_id'])) {
                $entity = $this->WizardAssignmentsTable->newEntity($wizardassignmentEntity);
                $this->WizardAssignmentsTable->save($entity);
            } else {
                $existingAssignment = $this->WizardAssignmentsTable->getWizardByUuidForEdit($wizardassignmentEntity['uuid']);
                foreach (array_diff($wizardassignmentEntity['servicetemplates']['_ids'], $existingAssignment['servicetemplates']['_ids']) as $missingServicetemplateId) {
                    $existingAssignment['servicetemplates']['_ids'][] = $missingServicetemplateId;
                }
                $entity = $this->WizardAssignmentsTable->get($existingAssignment['id']);
                $entity = $this->WizardAssignmentsTable->patchEntity($entity, $existingAssignment);
                $this->WizardAssignmentsTable->save($entity);
            }
        }
        return true;
    }

    /**
     * @return array
     */
    public function getData() {
        return [
            'Commands'         => $this->getCommandsData(),
            'Servicetemplates' => $this->getServicetemplatesData(),
            'Wizardassignment' => $this->getWizardAssignmentData()
        ];
    }

    /**
     * @return array
     */
    public function getCommandsData() {
        return [
            0 => [
                'name'             => 'check_oracle_health',
                'command_line'     => '$USER1$/check_oracle_health --connect=$ARG1$ --user=$_HOSTDBUSER$ --password=$_HOSTDBPASS$ --mode=$ARG2$ --warning=$ARG3$ --critical=$ARG4$ --ident --commit',
                'command_type'     => '1',
                'human_args'       => null,
                'uuid'             => '09b44de9-ce0a-449f-801b-14b7e15daccc',
                'description'      => '',
                'commandarguments' => [
                    0 => [
                        'name'       => '$ARG1$',
                        'human_name' => 'DB Name',
                    ],
                    1 => [
                        'name'       => '$ARG2$',
                        'human_name' => 'Mode',
                    ],
                    2 => [
                        'name'       => '$ARG3$',
                        'human_name' => 'Warning',
                    ],
                    3 => [
                        'name'       => '$ARG4$',
                        'human_name' => 'Critical',
                    ]
                ]
            ]
        ];
    }


    /**
     * @return array
     */
    public function getServicetemplatesData() {
        return [
            0 => [
                'uuid'                                      => 'aa7351ff-d3ea-406d-9ba4-4797b1ad50e0',
                'template_name'                             => 'ORACLE_CONNECTION-TIME',
                'name'                                      => 'Connection-Time DBNAME',
                'container_id'                              => '1',
                'servicetemplatetype_id'                    => '1',
                'check_period_id'                           => '1',
                'notify_period_id'                          => '1',
                'description'                               => '',
                'command_id'                                => '09b44de9-ce0a-449f-801b-14b7e15daccc',
                'check_command_args'                        => '',
                'checkcommand_info'                         => '',
                'eventhandler_command_id'                   => '0',
                'timeperiod_id'                             => null,
                'check_interval'                            => '300',
                'retry_interval'                            => '120',
                'max_check_attempts'                        => '3',
                'first_notification_delay'                  => '0',
                'notification_interval'                     => '7200',
                'notify_on_warning'                         => '1',
                'notify_on_unknown'                         => '1',
                'notify_on_critical'                        => '1',
                'notify_on_recovery'                        => '1',
                'notify_on_flapping'                        => '0',
                'notify_on_downtime'                        => '0',
                'flap_detection_enabled'                    => '0',
                'flap_detection_on_ok'                      => '0',
                'flap_detection_on_warning'                 => '0',
                'flap_detection_on_unknown'                 => '0',
                'flap_detection_on_critical'                => '0',
                'low_flap_threshold'                        => '0',
                'high_flap_threshold'                       => '0',
                'process_performance_data'                  => '1',
                'freshness_checks_enabled'                  => '0',
                'freshness_threshold'                       => '3600',
                'passive_checks_enabled'                    => '1',
                'event_handler_enabled'                     => '0',
                'active_checks_enabled'                     => '1',
                'retain_status_information'                 => '0',
                'retain_nonstatus_information'              => '0',
                'notifications_enabled'                     => '1',
                'notes'                                     => '',
                'priority'                                  => '1',
                'tags'                                      => '',
                'service_url'                               => '',
                'is_volatile'                               => '0',
                'check_freshness'                           => '0',
                'created'                                   => '2021-06-08 13:46:09',
                'modified'                                  => '2021-06-08 13:46:09',
                'servicetemplateeventcommandargumentvalues' => [],
                'servicetemplatecommandargumentvalues'      => [
                    0 => [
                        'commandargument_id' => '$ARG1$',
                        'value'              => 'DBNAME',
                    ],
                    1 => [
                        'commandargument_id' => '$ARG2$',
                        'value'              => 'connection-time',
                    ],
                    2 => [
                        'commandargument_id' => '$ARG3$',
                        'value'              => '2',
                    ],
                    3 =>
                        [
                            'commandargument_id' => '$ARG4$',
                            'value'              => '5',
                        ],
                ],
                'customvariables'                           => [],
                'servicegroups'                             => [],
                'contactgroups'                             => [],
                'contacts'                                  => [],
            ],
            1 => [
                'uuid'                                      => '24529c13-b583-48ea-b0be-8d9a53bfe168',
                'template_name'                             => 'ORACLE_CONNECTED_USERS',
                'name'                                      => 'Connected-Users DBNAME',
                'container_id'                              => '1',
                'servicetemplatetype_id'                    => '1',
                'check_period_id'                           => '1',
                'notify_period_id'                          => '1',
                'description'                               => '',
                'command_id'                                => '09b44de9-ce0a-449f-801b-14b7e15daccc',
                'check_command_args'                        => '',
                'checkcommand_info'                         => '',
                'eventhandler_command_id'                   => '0',
                'timeperiod_id'                             => null,
                'check_interval'                            => '300',
                'retry_interval'                            => '120',
                'max_check_attempts'                        => '3',
                'first_notification_delay'                  => '0',
                'notification_interval'                     => '7200',
                'notify_on_warning'                         => '1',
                'notify_on_unknown'                         => '1',
                'notify_on_critical'                        => '1',
                'notify_on_recovery'                        => '1',
                'notify_on_flapping'                        => '0',
                'notify_on_downtime'                        => '0',
                'flap_detection_enabled'                    => '0',
                'flap_detection_on_ok'                      => '0',
                'flap_detection_on_warning'                 => '0',
                'flap_detection_on_unknown'                 => '0',
                'flap_detection_on_critical'                => '0',
                'low_flap_threshold'                        => '0',
                'high_flap_threshold'                       => '0',
                'process_performance_data'                  => '1',
                'freshness_checks_enabled'                  => '0',
                'freshness_threshold'                       => '3600',
                'passive_checks_enabled'                    => '1',
                'event_handler_enabled'                     => '0',
                'active_checks_enabled'                     => '1',
                'retain_status_information'                 => '0',
                'retain_nonstatus_information'              => '0',
                'notifications_enabled'                     => '1',
                'notes'                                     => '',
                'priority'                                  => '1',
                'tags'                                      => '',
                'service_url'                               => '',
                'is_volatile'                               => '0',
                'check_freshness'                           => '0',
                'created'                                   => '2021-06-08 13:46:09',
                'modified'                                  => '2021-06-08 13:46:09',
                'servicetemplateeventcommandargumentvalues' => [],
                'servicetemplatecommandargumentvalues'      => [
                    0 => [
                        'commandargument_id' => '$ARG1$',
                        'value'              => 'DBNAME',
                    ],
                    1 => [
                        'commandargument_id' => '$ARG2$',
                        'value'              => 'connected-users',
                    ],
                    2 => [
                        'commandargument_id' => '$ARG3$',
                        'value'              => '50',
                    ],
                    3 => [
                        'commandargument_id' => '$ARG4$',
                        'value'              => '100',
                    ],
                ],
                'customvariables'                           => [],
                'servicegroups'                             => [],
                'contactgroups'                             => [],
                'contacts'                                  => [],
            ],
            2 => [
                'uuid'                                      => 'cdd0fcf1-d018-4601-a14d-70871f4ba341',
                'template_name'                             => 'ORACLE_SESSION_USAGE',
                'name'                                      => 'Session-Usage DBNAME',
                'container_id'                              => '1',
                'servicetemplatetype_id'                    => '1',
                'check_period_id'                           => '1',
                'notify_period_id'                          => '1',
                'description'                               => '',
                'command_id'                                => '09b44de9-ce0a-449f-801b-14b7e15daccc',
                'check_command_args'                        => '',
                'checkcommand_info'                         => '',
                'eventhandler_command_id'                   => '0',
                'timeperiod_id'                             => null,
                'check_interval'                            => '300',
                'retry_interval'                            => '120',
                'max_check_attempts'                        => '3',
                'first_notification_delay'                  => '0',
                'notification_interval'                     => '7200',
                'notify_on_warning'                         => '1',
                'notify_on_unknown'                         => '1',
                'notify_on_critical'                        => '1',
                'notify_on_recovery'                        => '1',
                'notify_on_flapping'                        => '0',
                'notify_on_downtime'                        => '0',
                'flap_detection_enabled'                    => '0',
                'flap_detection_on_ok'                      => '0',
                'flap_detection_on_warning'                 => '0',
                'flap_detection_on_unknown'                 => '0',
                'flap_detection_on_critical'                => '0',
                'low_flap_threshold'                        => '0',
                'high_flap_threshold'                       => '0',
                'process_performance_data'                  => '1',
                'freshness_checks_enabled'                  => '0',
                'freshness_threshold'                       => '3600',
                'passive_checks_enabled'                    => '1',
                'event_handler_enabled'                     => '0',
                'active_checks_enabled'                     => '1',
                'retain_status_information'                 => '0',
                'retain_nonstatus_information'              => '0',
                'notifications_enabled'                     => '1',
                'notes'                                     => '',
                'priority'                                  => '1',
                'tags'                                      => '',
                'service_url'                               => '',
                'is_volatile'                               => '0',
                'check_freshness'                           => '0',
                'created'                                   => '2021-06-08 13:46:09',
                'modified'                                  => '2021-06-08 13:46:09',
                'servicetemplateeventcommandargumentvalues' => [],
                'servicetemplatecommandargumentvalues'      => [
                    0 => [
                        'commandargument_id' => '$ARG1$',
                        'value'              => 'DBNAME',
                    ],
                    1 => [
                        'commandargument_id' => '$ARG2$',
                        'value'              => 'session-usage',
                    ],
                    2 => [
                        'commandargument_id' => '$ARG3$',
                        'value'              => '90',
                    ],
                    3 => [
                        'commandargument_id' => '$ARG4$',
                        'value'              => '95',
                    ],
                ],
                'customvariables'                           => [],
                'servicegroups'                             => [],
                'contactgroups'                             => [],
                'contacts'                                  => [],
            ],
            3 => [
                'uuid'                                      => 'd2b25746-2790-4257-b561-f05a1b5d6298',
                'template_name'                             => 'ORACLE_PROCESS_USAGE',
                'name'                                      => 'Process-Usage DBNAME',
                'container_id'                              => '1',
                'servicetemplatetype_id'                    => '1',
                'check_period_id'                           => '1',
                'notify_period_id'                          => '1',
                'description'                               => '',
                'command_id'                                => '09b44de9-ce0a-449f-801b-14b7e15daccc',
                'check_command_args'                        => '',
                'checkcommand_info'                         => '',
                'eventhandler_command_id'                   => '0',
                'timeperiod_id'                             => null,
                'check_interval'                            => '300',
                'retry_interval'                            => '120',
                'max_check_attempts'                        => '3',
                'first_notification_delay'                  => '0',
                'notification_interval'                     => '7200',
                'notify_on_warning'                         => '1',
                'notify_on_unknown'                         => '1',
                'notify_on_critical'                        => '1',
                'notify_on_recovery'                        => '1',
                'notify_on_flapping'                        => '0',
                'notify_on_downtime'                        => '0',
                'flap_detection_enabled'                    => '0',
                'flap_detection_on_ok'                      => '0',
                'flap_detection_on_warning'                 => '0',
                'flap_detection_on_unknown'                 => '0',
                'flap_detection_on_critical'                => '0',
                'low_flap_threshold'                        => '0',
                'high_flap_threshold'                       => '0',
                'process_performance_data'                  => '1',
                'freshness_checks_enabled'                  => '0',
                'freshness_threshold'                       => '3600',
                'passive_checks_enabled'                    => '1',
                'event_handler_enabled'                     => '0',
                'active_checks_enabled'                     => '1',
                'retain_status_information'                 => '0',
                'retain_nonstatus_information'              => '0',
                'notifications_enabled'                     => '1',
                'notes'                                     => '',
                'priority'                                  => '1',
                'tags'                                      => '',
                'service_url'                               => '',
                'is_volatile'                               => '0',
                'check_freshness'                           => '0',
                'created'                                   => '2021-06-08 13:46:09',
                'modified'                                  => '2021-06-08 13:46:09',
                'servicetemplateeventcommandargumentvalues' => [],
                'servicetemplatecommandargumentvalues'      => [
                    0 => [
                        'commandargument_id' => '$ARG1$',
                        'value'              => 'DBNAME',
                    ],
                    1 => [
                        'commandargument_id' => '$ARG2$',
                        'value'              => 'process-usage',
                    ],
                    2 => [
                        'commandargument_id' => '$ARG3$',
                        'value'              => '90',
                    ],
                    3 => [
                        'commandargument_id' => '$ARG4$',
                        'value'              => '95',
                    ],
                ],
                'customvariables'                           => [],
                'servicegroups'                             => [],
                'contactgroups'                             => [],
                'contacts'                                  => [],
            ],
            4 => [
                'uuid'                                      => '690e3d0d-d286-4561-941f-3a9ecc6fa9ea',
                'template_name'                             => 'ORACLE_RMAN_BACKUP',
                'name'                                      => 'RMAN Backup DBNAME',
                'container_id'                              => '1',
                'servicetemplatetype_id'                    => '1',
                'check_period_id'                           => '1',
                'notify_period_id'                          => '1',
                'description'                               => '',
                'command_id'                                => '09b44de9-ce0a-449f-801b-14b7e15daccc',
                'check_command_args'                        => '',
                'checkcommand_info'                         => '',
                'eventhandler_command_id'                   => '0',
                'timeperiod_id'                             => null,
                'check_interval'                            => '300',
                'retry_interval'                            => '120',
                'max_check_attempts'                        => '3',
                'first_notification_delay'                  => '0',
                'notification_interval'                     => '7200',
                'notify_on_warning'                         => '1',
                'notify_on_unknown'                         => '1',
                'notify_on_critical'                        => '1',
                'notify_on_recovery'                        => '1',
                'notify_on_flapping'                        => '0',
                'notify_on_downtime'                        => '0',
                'flap_detection_enabled'                    => '0',
                'flap_detection_on_ok'                      => '0',
                'flap_detection_on_warning'                 => '0',
                'flap_detection_on_unknown'                 => '0',
                'flap_detection_on_critical'                => '0',
                'low_flap_threshold'                        => '0',
                'high_flap_threshold'                       => '0',
                'process_performance_data'                  => '1',
                'freshness_checks_enabled'                  => '0',
                'freshness_threshold'                       => '3600',
                'passive_checks_enabled'                    => '1',
                'event_handler_enabled'                     => '0',
                'active_checks_enabled'                     => '1',
                'retain_status_information'                 => '0',
                'retain_nonstatus_information'              => '0',
                'notifications_enabled'                     => '1',
                'notes'                                     => '',
                'priority'                                  => '1',
                'tags'                                      => '',
                'service_url'                               => '',
                'is_volatile'                               => '0',
                'check_freshness'                           => '0',
                'created'                                   => '2021-06-08 13:46:09',
                'modified'                                  => '2021-06-08 13:46:09',
                'servicetemplateeventcommandargumentvalues' => [],
                'servicetemplatecommandargumentvalues'      => [
                    0 => [
                        'commandargument_id' => '$ARG1$',
                        'value'              => 'DBNAME',
                    ],
                    1 => [
                        'commandargument_id' => '$ARG2$',
                        'value'              => 'rman-backup-problems',
                    ],
                    2 => [
                        'commandargument_id' => '$ARG3$',
                        'value'              => '1',
                    ],
                    3 => [
                        'commandargument_id' => '$ARG4$',
                        'value'              => '2',
                    ],
                ],
                'customvariables'                           => [],
                'servicegroups'                             => [],
                'contactgroups'                             => [],
                'contacts'                                  => [],
            ],
            5 => [
                'uuid'                                      => 'a4655cbe-09ce-45e6-a57b-1d1918c52351',
                'template_name'                             => 'ORACLE_TABLESPACE_USAGE',
                'name'                                      => 'Tablespace-Usage DBNAME',
                'container_id'                              => '1',
                'servicetemplatetype_id'                    => '1',
                'check_period_id'                           => '1',
                'notify_period_id'                          => '1',
                'description'                               => '',
                'command_id'                                => '09b44de9-ce0a-449f-801b-14b7e15daccc',
                'check_command_args'                        => '',
                'checkcommand_info'                         => '',
                'eventhandler_command_id'                   => '0',
                'timeperiod_id'                             => null,
                'check_interval'                            => '300',
                'retry_interval'                            => '120',
                'max_check_attempts'                        => '3',
                'first_notification_delay'                  => '0',
                'notification_interval'                     => '7200',
                'notify_on_warning'                         => '1',
                'notify_on_unknown'                         => '1',
                'notify_on_critical'                        => '1',
                'notify_on_recovery'                        => '1',
                'notify_on_flapping'                        => '0',
                'notify_on_downtime'                        => '0',
                'flap_detection_enabled'                    => '0',
                'flap_detection_on_ok'                      => '0',
                'flap_detection_on_warning'                 => '0',
                'flap_detection_on_unknown'                 => '0',
                'flap_detection_on_critical'                => '0',
                'low_flap_threshold'                        => '0',
                'high_flap_threshold'                       => '0',
                'process_performance_data'                  => '1',
                'freshness_checks_enabled'                  => '0',
                'freshness_threshold'                       => '3600',
                'passive_checks_enabled'                    => '1',
                'event_handler_enabled'                     => '0',
                'active_checks_enabled'                     => '1',
                'retain_status_information'                 => '0',
                'retain_nonstatus_information'              => '0',
                'notifications_enabled'                     => '1',
                'notes'                                     => '',
                'priority'                                  => '1',
                'tags'                                      => '',
                'service_url'                               => '',
                'is_volatile'                               => '0',
                'check_freshness'                           => '0',
                'created'                                   => '2021-06-08 13:46:09',
                'modified'                                  => '2021-06-08 13:46:09',
                'servicetemplateeventcommandargumentvalues' => [],
                'servicetemplatecommandargumentvalues'      => [
                    0 => [
                        'commandargument_id' => '$ARG1$',
                        'value'              => 'DBNAME',
                    ],
                    1 => [
                        'commandargument_id' => '$ARG2$',
                        'value'              => 'tablespace-usage',
                    ],
                    2 => [
                        'commandargument_id' => '$ARG3$',
                        'value'              => '90',
                    ],
                    3 => [
                        'commandargument_id' => '$ARG4$',
                        'value'              => '95',
                    ],
                ],
                'customvariables'                           => [],
                'servicegroups'                             => [],
                'contactgroups'                             => [],
                'contacts'                                  => [],
            ]
        ];
    }


    /**
     * @param string $name
     * @param array $commandarguments
     * @return bool|mixed
     */
    public function getCommandArgumentIdByName(string $name, array $commandarguments) {
        foreach ($commandarguments as $commandargument) {
            if (isset($commandargument['name']) && $commandargument['name'] === $name) {
                return $commandargument['id'];
            }
        }
        return false;
    }

    /**
     * @return array
     */
    public function getWizardAssignmentData() {
        $data = [

            'uuid'             => '7c5177a1-ff34-4a12-922b-2086211b62db',
            'type_id'          => 'oracle',
            'servicetemplates' => [
                '_ids' => [
                    'aa7351ff-d3ea-406d-9ba4-4797b1ad50e0',
                    '24529c13-b583-48ea-b0be-8d9a53bfe168',
                    'cdd0fcf1-d018-4601-a14d-70871f4ba341',
                    'd2b25746-2790-4257-b561-f05a1b5d6298',
                    '690e3d0d-d286-4561-941f-3a9ecc6fa9ea',
                    'a4655cbe-09ce-45e6-a57b-1d1918c52351'
                ]
            ]
        ];
        return $data;
    }
}
